/*-------------------------------------------------------------------------+
| Copyright 2021 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editor.fx.visual;

import static org.fortiss.tooling.base.ui.utils.LWFXEditorUtils.stickyConnectorHasVisibleConnections;

import org.fortiss.tooling.base.model.element.IConnector;
import org.fortiss.tooling.common.ui.javafx.lwfxef.DiagramLayers;
import org.fortiss.tooling.common.ui.javafx.lwfxef.model.layout.ISideLayout;
import org.fortiss.tooling.common.ui.javafx.lwfxef.visual.IAnchorageVisual;
import org.fortiss.tooling.common.ui.javafx.lwfxef.visual.widgets.AlignedText;

import javafx.geometry.Pos;
import javafx.geometry.Rectangle2D;

/**
 * Manages visual node(s) for the display of anchorage text labels.
 */
/* package */ class AnchorageText {

	/** The text displaying the anchorage name */
	AlignedText nameText;

	/** Constructor */
	public AnchorageText() {
		this.nameText = new AlignedText();
		nameText.setMouseTransparent(true);
	}

	/**
	 * Update the text nodes with the data from the given anchorage visual,
	 * the model element and the given text. Place the nodes on the given layers.
	 */
	public <V extends IAnchorageVisual & ISideLayout> //
	void updateNodes(V visual, IConnector connector, String text, DiagramLayers layers) {
		nameText.setText(text);
		updateNameTextPosition(visual, connector);
		if(nameText.getParent() == null) {
			layers.getVisualFeedbackLayer().add(nameText, visual.getMVCBundle());
		}
	}

	/**
	 * Sets the position of the text.
	 * Depending on the side, the text is placed off to the side.
	 */
	private <V extends IAnchorageVisual & ISideLayout> //
	void updateNameTextPosition(V visual, IConnector connector) {
		Rectangle2D bounds = visual.getCurrentBounds();
		switch(visual.getSide()) {
			case LEFT:
				if(stickyConnectorHasVisibleConnections(connector)) {
					// move the text a little to the side if there is a connection
					nameText.setX(bounds.getMinX() + bounds.getWidth() / 4);
					nameText.setY(bounds.getMinY() + bounds.getHeight() / 4);
					nameText.setAlignment(Pos.BOTTOM_RIGHT);
				} else {
					nameText.setX(bounds.getMinX());
					nameText.setY(bounds.getMinY() + bounds.getHeight() / 2);
					nameText.setAlignment(Pos.CENTER_RIGHT);
				}
				break;
			case RIGHT:
				if(stickyConnectorHasVisibleConnections(connector)) {
					// move the text a little to the side if there is a connection
					nameText.setX(bounds.getMaxX() - bounds.getWidth() / 4);
					nameText.setY(bounds.getMinY() + bounds.getHeight() / 4);
					nameText.setAlignment(Pos.BOTTOM_LEFT);
				} else {
					nameText.setX(bounds.getMaxX());
					nameText.setY(bounds.getMinY() + bounds.getHeight() / 2);
					nameText.setAlignment(Pos.CENTER_LEFT);
				}
				break;
			case TOP:
				nameText.setX(bounds.getMaxX());
				nameText.setY(bounds.getMinY());
				nameText.setAlignment(Pos.BOTTOM_CENTER);
				break;
			case BOTTOM:
				nameText.setX(bounds.getMaxX());
				nameText.setY(bounds.getMaxY());
				nameText.setAlignment(Pos.TOP_CENTER);
				break;
		}
	}

	/** Remove the text from layers; do nothing if the text is not placed on any node. */
	public void removeAllVisuals(DiagramLayers layers) {
		if(nameText.getParent() != null) {
			layers.getVisualFeedbackLayer().remove(nameText);
		}
	}
}
